package org.apache.poi.xssf.usermodel;

import org.apache.poi.ss.usermodel.Hyperlink;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

/**
 * @see <a href="https://issues.apache.org/bugzilla/show_bug.cgi?id=56527">bug 56527</a>
 * @throws Exception
 */
public class TestCreateMultiCellHyperlink {

	private static final String URL = "http://www.multicell.com/";

	// B3
	private static final int FIRST_ROW = 2;
	private static final int FIRST_COL = 1;
    
    // D4
	private static final int LAST_ROW =  3;
	private static final int LAST_COL =  3;
		
	private XSSFSheet ws;
	private XSSFHyperlink hyperlink;
		
	@Before
	public void setUp() {
		XSSFWorkbook wb = new XSSFWorkbook();
        ws = wb.createSheet();
	        
        // Creo un'area di celle 5x3 (A1:E5)
        for (int r = 0; r < 5; r++) {
            XSSFRow row = ws.createRow(r);
            for (int c = 1; c < 5; c++) {
                row.createCell(c);
            }
        }
	        
        XSSFCreationHelper creationHelper = wb.getCreationHelper();
        hyperlink = creationHelper.createHyperlink(Hyperlink.LINK_URL);
        hyperlink.setAddress(URL);
        ws.addHyperlink(hyperlink);
	}
	
	@Test
    public void testCreateMuliCellHyperlink1() {        
        hyperlink.setFirstRow(FIRST_ROW);
        hyperlink.setFirstColumn(FIRST_COL);
        hyperlink.setLastRow(LAST_ROW);
        hyperlink.setLastColumn(LAST_COL);
        hyperlink.setAddress(URL);
        
        doChecks();
    }
	
	@Test
    public void testCreateMuliCellHyperlink2() {        
		hyperlink.setFirstColumn(FIRST_COL);
        hyperlink.setFirstRow(FIRST_ROW);
        hyperlink.setLastColumn(LAST_COL);
        hyperlink.setLastRow(LAST_ROW);        
        
        doChecks();
    }

	@Test
    public void testCreateMuliCellHyperlink3() {        
        hyperlink.setLastRow(LAST_ROW);
        hyperlink.setLastColumn(LAST_COL);
        hyperlink.setFirstRow(FIRST_ROW);
        hyperlink.setFirstColumn(FIRST_COL);

        doChecks();
    }
	
	@Test
    public void testCreateMuliCellHyperlink4() {        
        hyperlink.setLastColumn(LAST_COL);
        hyperlink.setLastRow(LAST_ROW);        
		hyperlink.setFirstColumn(FIRST_COL);
        hyperlink.setFirstRow(FIRST_ROW);

        doChecks();
    }

	
	private void doChecks() {
		for (int r = FIRST_ROW; r <= LAST_ROW; r++) {
            XSSFRow row = ws.getRow(r);
            for(int c = FIRST_COL; c <= LAST_COL; c++) {
                XSSFCell cell = row.getCell(c);
                XSSFHyperlink hyp = cell.getHyperlink();
                assertNotNull("Atteso hyperlink in (" + r + "," + c +")", hyp);
                
                assertEquals("B3:D4", hyp.getCellRef());
                
                // Check dei metodi getFistRow(), getFirstColumn(), getLastRow(), getLastColumn()
                assertEquals(FIRST_ROW, hyperlink.getFirstRow());
                assertEquals(FIRST_COL, hyperlink.getFirstColumn());
                assertEquals(LAST_ROW, hyperlink.getLastRow());                
                assertEquals(LAST_COL, hyperlink.getLastColumn());
                                
                assertEquals(URL, hyp.getAddress());
            }
        }
	}
}
