package org.apache.poi.xssf.usermodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.InputStream;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Hyperlink;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.junit.Test;

/**
 * @see <a href="https://issues.apache.org/bugzilla/show_bug.cgi?id=55904">bug 55904</a>
 * @throws Exception
 */
public class TestMultiCellHyperlink {
    
    private static final String URL = "http://www.multicell.com/";
    
    @Test
    public void testGetHyperlink() throws Exception {
        InputStream is = this.getClass().getClassLoader().getResourceAsStream("org/apache/poi/xssf/usermodel/TestMultiCellHyperlink.xlsx");
        Workbook wb = WorkbookFactory.create(is);
        Sheet ws = wb.getSheetAt(0);
        // Test di controllo --> hyperlink a "singola cella"
        Cell cell = ws.getRow(6).getCell(1); //B7
        Hyperlink hyperlink = cell.getHyperlink();
        
        // Check dei metodi getFistRow(), getFirstColumn(), getLastRow(), getLastColumn()
        assertEquals(6, hyperlink.getFirstRow());
        assertEquals(1, hyperlink.getFirstColumn());
        assertEquals(6, hyperlink.getLastRow());
        assertEquals(1, hyperlink.getLastColumn());
        
        assertNotNull(hyperlink);
        assertEquals("http://www.singlecell.com/", hyperlink.getAddress());
        
        // Test di stesso hyperlink presente su piu'celle
        // B3
        int firstRow = 2;
        int firstCol = 1;
        
        // D4
        int lastRow =  3;
        int lastCol =  3;
        
        for (int r = firstRow; r <= lastRow; r++) {
            Row row = ws.getRow(r);
            for(int c = firstCol; c <= lastCol; c++) {
                cell = row.getCell(c);
                hyperlink = cell.getHyperlink();                
                assertNotNull("Atteso hyperlink in (" + r + "," + c +")", hyperlink);
                
                // Check dei metodi getFistRow(), getFirstColumn(), getLastRow(), getLastColumn()
                assertEquals(firstRow, hyperlink.getFirstRow());
                assertEquals(firstCol, hyperlink.getFirstColumn());
                assertEquals(lastRow, hyperlink.getLastRow());
                assertEquals(lastCol, hyperlink.getLastColumn());
                
                assertEquals(URL, hyperlink.getAddress());
            }
        }
        
        // Test di controllo su assenza di hyperlink
        checkNull(ws, firstRow - 1, firstCol);
        checkNull(ws, firstRow, firstCol - 1);
        checkNull(ws,  lastRow + 1, lastCol);
        checkNull(ws,  lastRow, lastCol + 1);       
    }
    
    private void checkNull(Sheet ws, int row, int col) {
        Row r = ws.getRow(row);
        if (r == null) {
            r = ws.createRow(row);
        }
        Cell cell = r.getCell(col);
        if (cell == null) {
            cell = r.createCell(col);
        }
        Hyperlink hyperlink = cell.getHyperlink();
        assertNull(hyperlink);
    }       
}
